@@ -226,8 +226,8 @@ KDE_NO_EXPORT void Settings::applyColorSetting (bool only_changed_ones) {
             QPalette palette;
             switch (ColorSetting::Target (i)) {
                 case ColorSetting::playlist_background:
-                   palette.setColor (view->playList()->backgroundRole(), colors[i].color);
-                   view->playList()->setPalette (palette);
+                   palette.setColor (view->playList()->viewport ()->backgroundRole(), colors[i].color);
+                   view->playList()->viewport ()->setPalette (palette);
                    break;
                 case ColorSetting::playlist_foreground:
                    palette.setColor (view->playList()->foregroundRole(), colors[i].color);
@@ -634,7 +634,7 @@ void Settings::okPressed () {
                     urlchanged = false;
                     KMessageBox::error (m_player->view (), i18n ("File %1 does not exist.",url.url ()), i18n ("Error"));
                 } else {
-                    configdialog->m_SourcePageURL->url->setUrl (fi.absFilePath () + xine_directives);
+                    configdialog->m_SourcePageURL->url->setUrl (QString(fi.absFilePath () + xine_directives));
                 }
             }
             if (urlchanged &&
@@ -738,8 +738,7 @@ void Settings::okPressed () {
 #else
     recorder = Recorder (configdialog->m_RecordPage->recorder->selectedId ());
 #endif
-    replaytime = configdialog->m_RecordPage->replaytime->text ().toInt ();
-    configdialog->m_RecordPage->replaytime->setValue (replaytime);
+    replaytime = configdialog->m_RecordPage->replaytime->value ();
     recordfile = configdialog->m_RecordPage->url->lineEdit()->text ();
     mencoderarguments = configdialog->m_MEncoderPage->arguments->text ();
     ffmpegarguments = configdialog->m_FFMpegPage->arguments->text ();
