@@ -609,10 +609,13 @@ inline Node *findChildWithId (const Node *p, const short id) {
 
 class KMPLAYER_EXPORT PlaylistRole {
 public:
+    PlaylistRole () : editable (false) {}
+
     QString caption () const;
     void setCaption (const QString &t);
 
     QString title;
+    bool editable;
 };
 
 /**
@@ -695,11 +698,12 @@ public:
  **/
 class KMPLAYER_EXPORT Posting {
 public:
-    KDE_NO_CDTOR_EXPORT Posting (Node *n, MessageType msg)
-        : source (n), message (msg) {}
+    KDE_NO_CDTOR_EXPORT Posting (Node *n, MessageType msg, VirtualVoid *p=NULL)
+        : source (n), message (msg), payload (p) {}
     KDE_NO_CDTOR_EXPORT virtual ~Posting () {}
     NodePtrW source;
     MessageType message;
+    VirtualVoid *payload;
 };
 
 /**
@@ -824,12 +828,12 @@ namespace SMIL {
     class Transition;
     class AnimateBase;
     class MediaType;
-    class ImageMediaType;
     class TextMediaType;
     class RefMediaType;
     class Brush;
     class SmilText;
     class TextFlow;
+    class TemporalMoment;
     class PriorityClass;
     class Anchor;
     class Area;
@@ -863,12 +867,12 @@ public:
     virtual void visit (SMIL::AnimateBase *);
     virtual void visit (SMIL::PriorityClass *);
     virtual void visit (SMIL::MediaType *);
-    virtual void visit (SMIL::ImageMediaType *);
     virtual void visit (SMIL::TextMediaType *);
     virtual void visit (SMIL::RefMediaType *);
     virtual void visit (SMIL::Brush *);
     virtual void visit (SMIL::SmilText *);
     virtual void visit (SMIL::TextFlow *);
+    virtual void visit (SMIL::TemporalMoment *);
     virtual void visit (SMIL::Anchor *);
     virtual void visit (SMIL::Area *);
     virtual void visit (RP::Imfl *) {}
