@@ -38,6 +38,7 @@
 #include <QtDBus/QtDBus>
 #include <QtCore/QDir>
 #include <QtCore/QUrl>
+#include <QNetworkCookie>
 
 #include <kdebug.h>
 #include <kprotocolmanager.h>
@@ -48,6 +49,7 @@
 #include <kstandarddirs.h>
 #include <kshell.h>
 #include <kio/job.h>
+#include <kio/accessmanager.h>
 
 #include "kmplayer_def.h"
 #include "kmplayerconfig.h"
@@ -105,6 +107,26 @@ static QString getPath (const KUrl & url) {
     return p;
 }
 
+static QString encodeFileOrUrl (const KUrl &url)
+{
+    return url.isEmpty ()
+        ? QString ()
+        : QString::fromLocal8Bit (QFile::encodeName (
+                    url.isLocalFile ()
+                    ? url.toLocalFile ()
+                    : QUrl::fromPercentEncoding (url.url ().toLocal8Bit ())));
+}
+
+static QString encodeFileOrUrl (const QString &str)
+{
+    if (!str.startsWith (QString ("dvd:")) &&
+            !str.startsWith (QString ("vcd:")) &&
+            !str.startsWith (QString ("tv:")) &&
+            !str.startsWith (QString ("cdda:")))
+        return encodeFileOrUrl (KUrl (str));
+    return str;
+}
+
 static void setupProcess (QProcess **process)
 {
     delete *process;
@@ -195,6 +217,9 @@ void Process::setSubtitle (int, const QString &) {}
 void Process::pause () {
 }
 
+void Process::unpause () {
+}
+
 bool Process::seek (int /*pos*/, bool /*absolute*/) {
     return false;
 }
@@ -267,7 +292,10 @@ bool Process::play () {
     m_url = url;
     if (user) // FIXME: remove check
         user->starting (this);
-    if (!changed || KUrl (m_url).isLocalFile () || nonstdurl)
+    if (!changed ||
+            KUrl (m_url).isLocalFile () ||
+            nonstdurl ||
+            (m_source && m_source->avoidRedirects ()))
         return deMediafiedPlay ();
     m_job = KIO::stat (m_url, KIO::HideProgressInfo);
     connect (m_job, SIGNAL (result (KJob *)), this, SLOT (result (KJob *)));
@@ -395,6 +423,16 @@ KDE_NO_EXPORT void MPlayerBase::initProcess () {
             this, SLOT (processStopped (int, QProcess::ExitStatus)));
 }
 
+bool MPlayerBase::removeQueued (const char *cmd) {
+    const QList<QByteArray>::iterator e = commands.end ();
+    for (QList<QByteArray>::iterator i = commands.begin (); i != e; ++i)
+        if (!strncmp ((*i).data (), cmd, strlen (cmd))) {
+            commands.erase (i);
+            return true;
+        }
+    return false;
+}
+
 KDE_NO_EXPORT bool MPlayerBase::sendCommand (const QString & cmd) {
     if (running ()) {
         commands.push_front (QString (cmd + '\n').toAscii ());
@@ -435,13 +473,13 @@ KDE_NO_EXPORT void MPlayerBase::dataWritten (qint64) {
 }
 
 KDE_NO_EXPORT void MPlayerBase::processStopped () {
+    setState (IProcess::Ready);
 }
 
 KDE_NO_EXPORT void MPlayerBase::processStopped (int, QProcess::ExitStatus) {
     kDebug() << "process stopped" << endl;
     commands.clear ();
     processStopped ();
-    setState (IProcess::Ready);
 }
 
 //-----------------------------------------------------------------------------
@@ -466,6 +504,7 @@ KDE_NO_CDTOR_EXPORT
 MPlayer::MPlayer (QObject *parent, ProcessInfo *pinfo, Settings *settings)
  : MPlayerBase (parent, pinfo, settings, "mplayer"),
    m_widget (0L),
+   m_transition_state (NotRunning),
    aid (-1), sid (-1)
 {}
 
@@ -487,6 +526,8 @@ KDE_NO_EXPORT bool MPlayer::ready () {
 KDE_NO_EXPORT bool MPlayer::deMediafiedPlay () {
     if (running ())
         return sendCommand (QString ("gui_play"));
+
+    m_transition_state = NotRunning;
     if (!m_needs_restarted && running ())
         quit (); // rescheduling of setState will reset state just-in-time
 
@@ -499,11 +540,19 @@ KDE_NO_EXPORT bool MPlayer::deMediafiedPlay () {
     m_process_output = QString ();
     m_source->setPosition (0);
     if (!m_needs_restarted) {
-        aid = sid = -1;
-    } else
+        if (m_source->identified ()) {
+            aid = m_source->audioLangId ();
+            sid = m_source->subTitleId ();
+        } else {
+            aid = sid = -1;
+        }
+    } else {
         m_needs_restarted = false;
-    alanglist = 0L;
-    slanglist = 0L;
+    }
+    alanglist = NULL;
+    slanglist = NULL;
+    slanglist_end = NULL;
+    alanglist_end = NULL;
     m_request_seek = -1;
     m_tmpURL.truncate (0);
 
@@ -519,15 +568,15 @@ KDE_NO_EXPORT bool MPlayer::deMediafiedPlay () {
 
     QString strVideoDriver = QString (m_settings->videodrivers[m_settings->videodriver].driver);
     if (!strVideoDriver.isEmpty ()) {
-        args << "-vo" << strVideoDriver.lower();
+        args << "-vo" << strVideoDriver.toLower();
         if (view () && view ()->keepSizeRatio () &&
-                strVideoDriver.lower() == QString::fromLatin1 ("x11"))
+                strVideoDriver.toLower() == QString::fromLatin1 ("x11"))
             args << "-zoom";
     }
 
     QString strAudioDriver = QString (m_settings->audiodrivers[m_settings->audiodriver].driver);
     if (!strAudioDriver.isEmpty ())
-        args << "-ao" << strAudioDriver.lower();
+        args << "-ao" << strAudioDriver.toLower();
 
     if (m_settings->framedrop)
         args << "-framedrop";
@@ -571,10 +620,10 @@ KDE_NO_EXPORT bool MPlayer::deMediafiedPlay () {
             m_process->setWorkingDirectory
                 (QFileInfo (m_source->url ().path ()).dirPath (true));
         if (url.isLocalFile ()) {
-            m_url = getPath (url);
+            m_url = url.toLocalFile ();
             if (cfg_page->alwaysbuildindex &&
-                    (m_url.lower ().endsWith (".avi") ||
-                     m_url.lower ().endsWith (".divx")))
+                    (m_url.toLower ().endsWith (".avi") ||
+                     m_url.toLower ().endsWith (".divx")))
                 args << "-idx";
         } else {
             int cache = cfg_page->cachesize;
@@ -587,22 +636,17 @@ KDE_NO_EXPORT bool MPlayer::deMediafiedPlay () {
                 m_url = QString ("cdda://") + m_url.mid (6);
         }
         if (url.protocol () != QString ("stdin"))
-            args << QString (QFile::encodeName (m_url));
+            args << encodeFileOrUrl (m_url);
     }
-        Mrl *m = mrl ();
-        if (m && m->repeat > 0)
-            args << "-loop" << QString::number (m->repeat);
-        else if (m_settings->loop)
-            args << "-loop" << 0;
-            args << "-identify";
-        if (!m_source->subUrl ().isEmpty ()) {
-            args << "-sub";
-            const KUrl & sub_url (m_source->subUrl ());
-            if (!sub_url.isEmpty ()) {
-                QString myurl (sub_url.isLocalFile () ? getPath (sub_url) : sub_url.url ());
-                args << QString (QFile::encodeName (myurl));
-            }
-        }
+    Mrl *m = mrl ();
+    if (m && m->repeat > 0)
+        args << "-loop" << QString::number (m->repeat);
+    else if (m_settings->loop)
+        args << "-loop" << 0;
+    args << "-identify";
+    const QString surl = encodeFileOrUrl (m_source->subUrl ());
+    if (!surl.isEmpty ())
+        args << "-sub" << surl;
     qDebug ("mplayer %s\n", args.join (" ").toLocal8Bit ().constData ());
 
     startProcess (exe, args);
@@ -621,7 +665,21 @@ KDE_NO_EXPORT void MPlayer::stop () {
 }
 
 KDE_NO_EXPORT void MPlayer::pause () {
-    sendCommand (QString ("pause"));
+    if (Paused != m_transition_state) {
+        m_transition_state = Paused;
+        if (!removeQueued ("pause"))
+            sendCommand (QString ("pause"));
+    }
+}
+
+KDE_NO_EXPORT void MPlayer::unpause () {
+    if (m_transition_state == Paused
+            || (Paused == m_state
+                && m_transition_state != Playing)) {
+        m_transition_state = Playing;
+        if (!removeQueued ("pause"))
+            sendCommand (QString ("pause"));
+    }
 }
 
 KDE_NO_EXPORT bool MPlayer::seek (int pos, bool absolute) {
@@ -690,13 +748,11 @@ KDE_NO_EXPORT bool MPlayer::grabPicture (const QString &file, int pos) {
         return false; //FIXME
     initProcess ();
     m_old_state = m_state = Buffering;
-    unlink (file.ascii ());
+    unlink (file.toAscii ().constData ());
     QByteArray ba = file.toLocal8Bit ();
     ba.append ("XXXXXX");
     if (mkdtemp ((char *) ba.constData ())) {
         m_grab_dir = QString::fromLocal8Bit (ba.constData ());
-        KUrl url (m->src);
-        QString myurl (url.isLocalFile () ? getPath (url) : url.url ());
         QString exe ("mplayer");
         QStringList args;
         QString jpgopts ("jpeg:outdir=");
@@ -705,7 +761,7 @@ KDE_NO_EXPORT bool MPlayer::grabPicture (const QString &file, int pos) {
         args << "-frames" << "1" << "-nosound" << "-quiet";
         if (pos > 0)
             args << "-ss" << QString::number (pos);
-        args << QString (QFile::encodeName (myurl));
+        args << encodeFileOrUrl (m->src);
         kDebug () << args.join (" ");
         m_process->start (exe, args);
         if (m_process->waitForStarted ()) {
@@ -758,10 +814,16 @@ KDE_NO_EXPORT void MPlayer::processOutput () {
         if (process_stats) {
             QRegExp & m_posRegExp = patterns[MPlayerPreferencesPage::pat_pos];
             QRegExp & m_cacheRegExp = patterns[MPlayerPreferencesPage::pat_cache];
-            if (m_source->hasLength () && m_posRegExp.search (out) > -1) {
-                int pos = int (10.0 * m_posRegExp.cap (1).toFloat ());
-                m_source->setPosition (pos);
-                m_request_seek = -1;
+            if (m_posRegExp.search (out) > -1) {
+                if (m_source->hasLength ()) {
+                    int pos = int (10.0 * m_posRegExp.cap (1).toFloat ());
+                    m_source->setPosition (pos);
+                    m_request_seek = -1;
+                }
+                if (Playing == m_transition_state) {
+                    m_transition_state = NotRunning;
+                    setState (Playing);
+                }
             } else if (m_cacheRegExp.search (out) > -1) {
                 m_source->setLoading (int (m_cacheRegExp.cap(1).toDouble()));
             }
@@ -773,12 +835,18 @@ KDE_NO_EXPORT void MPlayer::processOutput () {
                     m_source->setLength (mrl (), 10 * l);
                 }
             }
+        } else if (out.startsWith ("ID_PAUSED")) {
+            if (Paused == m_transition_state) {
+                m_transition_state = NotRunning;
+                setState (Paused);
+            }
         } else if (m_refURLRegExp.search(out) > -1) {
             kDebug () << "Reference mrl " << m_refURLRegExp.cap (1);
             if (!m_tmpURL.isEmpty () &&
                     (m_url.endsWith (m_tmpURL) || m_tmpURL.endsWith (m_url)))
                 m_source->insertURL (mrl (), m_tmpURL);;
-            m_tmpURL = KUrl (m_refURLRegExp.cap (1)).url ();
+            KUrl tmp (m_refURLRegExp.cap (1));
+            m_tmpURL = tmp.isLocalFile () ? tmp.toLocalFile () : tmp.url ();
             if (m_source->url () == m_tmpURL ||
                     m_url.endsWith (m_tmpURL) || m_tmpURL.endsWith (m_url))
                 m_tmpURL.truncate (0);
@@ -817,10 +885,10 @@ KDE_NO_EXPORT void MPlayer::processOutput () {
                 pos = out.find ('=', pos);
                 if (pos > 0) {
                     if (!alanglist_end) {
-                        alanglist = new LangInfo (id, out.mid (pos + 1));
+                        alanglist = new Source::LangInfo (id, out.mid (pos + 1));
                         alanglist_end = alanglist;
                     } else {
-                        alanglist_end->next = new LangInfo (id, out.mid(pos+1));
+                        alanglist_end->next = new Source::LangInfo (id, out.mid(pos+1));
                         alanglist_end = alanglist_end->next;
                     }
                     kDebug () << "lang " << id << " " << alanglist_end->name;
@@ -833,10 +901,10 @@ KDE_NO_EXPORT void MPlayer::processOutput () {
                 pos = out.find ('=', pos);
                 if (pos > 0) {
                     if (!slanglist_end) {
-                        slanglist = new LangInfo (id, out.mid (pos + 1));
+                        slanglist = new Source::LangInfo (id, out.mid (pos + 1));
                         slanglist_end = slanglist;
                     } else {
-                        slanglist_end->next = new LangInfo (id, out.mid(pos+1));
+                        slanglist_end->next = new Source::LangInfo (id, out.mid(pos+1));
                         slanglist_end = slanglist_end->next;
                     }
                     kDebug () << "sid " << id << " " << slanglist_end->name;
@@ -871,12 +939,7 @@ KDE_NO_EXPORT void MPlayer::processOutput () {
                         m_tmpURL.truncate (0);
                     }
                     m_source->setIdentified ();
-                    QStringList alst, slst;
-                    for (SharedPtr <LangInfo> li = alanglist; li; li = li->next)
-                        alst.push_back (li->name);
-                    for (SharedPtr <LangInfo> li = slanglist; li; li = li->next)
-                        slst.push_back (li->name);
-                    m_source->setLanguages (alst, slst);
+                    m_source->setLanguages (alanglist, slanglist);
                     setState (IProcess::Playing);
                     m_source->setPosition (0);
                 }
@@ -916,26 +979,20 @@ KDE_NO_EXPORT void MPlayer::processStopped () {
             int pos = m_source->position ();
             play ();
             seek (pos, true);
+            return;
         }
     }
+    setState (IProcess::Ready);
 }
 
 void MPlayer::setAudioLang (int id, const QString &) {
-    SharedPtr <LangInfo> li = alanglist;
-    for (; id > 0 && li; li = li->next)
-        id--;
-    if (li)
-        aid = li->id;
+    aid = id;
     m_needs_restarted = true;
     sendCommand (QString ("quit"));
 }
 
 void MPlayer::setSubtitle (int id, const QString &) {
-    SharedPtr <LangInfo> li = slanglist;
-    for (; id > 0 && li; li = li->next)
-        id--;
-    if (li)
-        sid = li->id;
+    sid = id;
     m_needs_restarted = true;
     sendCommand (QString ("quit"));
 }
@@ -962,10 +1019,6 @@ static struct MPlayerPattern {
     { i18n ("Reference URL pattern"), "Reference URL Pattern", "Playing\\s+(.*[^\\.])\\.?\\s*$" },
     { i18n ("Reference pattern"), "Reference Pattern", "Reference Media file" },
     { i18n ("Start pattern"), "Start Playing", "Start[^ ]* play" },
-    { i18n ("DVD language pattern"), "DVD Language", "\\[open].*audio.*language: ([A-Za-z]+).*aid.*[^0-9]([0-9]+)" },
-    { i18n ("DVD subtitle pattern"), "DVD Sub Title", "\\[open].*subtitle.*[^0-9]([0-9]+).*language: ([A-Za-z]+)" },
-    { i18n ("DVD titles pattern"), "DVD Titles", "There are ([0-9]+) titles" },
-    { i18n ("DVD chapters pattern"), "DVD Chapters", "There are ([0-9]+) chapters" },
     { i18n ("VCD track pattern"), "VCD Tracks", "track ([0-9]+):" },
     { i18n ("Audio CD tracks pattern"), "CDROM Tracks", "[Aa]udio CD[^0-9]+([0-9]+)[^0-9]tracks" }
 };
@@ -1101,13 +1154,11 @@ KDE_NO_EXPORT void MEncoder::init () {
 }
 
 bool MEncoder::deMediafiedPlay () {
-    bool success = false;
     stop ();
     RecordDocument *rd = recordDocument (user);
     if (!rd)
         return false;
     initProcess ();
-    KUrl url (m_url);
     QString exe ("mencoder");
     QString margs = m_settings->mencoderarguments;
     if (m_settings->recordcopy)
@@ -1117,13 +1168,10 @@ bool MEncoder::deMediafiedPlay () {
         args << KShell::splitArgs (m_source->recordCmd ());
     // FIXME if (m_player->source () == source) // ugly
     //    m_player->stop ();
-    QString myurl = url.isLocalFile () ? getPath (url) : url.url ();
-    if (!myurl.isEmpty ()) {
-        args << QString (QFile::encodeName (myurl));
-    }
-    KUrl out (rd->record_file);
-    args << "-o" << QString (QFile::encodeName (
-                out.isLocalFile () ? getPath (out) : out.url ()));
+    QString myurl = encodeFileOrUrl (m_url);
+    if (!myurl.isEmpty ())
+        args << myurl;
+    args << "-o" << encodeFileOrUrl (rd->record_file);
     startProcess (exe, args);
     qDebug ("mencoder %s\n", args.join (" ").toLocal8Bit ().constData ());
     if (m_process->waitForStarted ()) {
@@ -1172,26 +1220,20 @@ KDE_NO_EXPORT void MPlayerDumpstream::init () {
 }
 
 bool MPlayerDumpstream::deMediafiedPlay () {
-    bool success = false;
     stop ();
     RecordDocument *rd = recordDocument (user);
     if (!rd)
         return false;
     initProcess ();
-    KUrl url (m_url);
     QString exe ("mplayer");
     QStringList args;
     args << KShell::splitArgs (m_source->recordCmd ());
     // FIXME if (m_player->source () == source) // ugly
     //    m_player->stop ();
-    QString myurl = url.isLocalFile () ? getPath (url) : url.url ();
-    if (!myurl.isEmpty ()) {
-        args << QString (QFile::encodeName (myurl));
-    }
-    KURL out (rd->record_file);
-    args << "-dumpstream" << "-dumpfile" <<
-        QString (QFile::encodeName (out.isLocalFile ()
-                    ? getPath (out) : out.url ()));
+    QString myurl = encodeFileOrUrl (m_url);
+    if (!myurl.isEmpty ())
+        args << myurl;
+    args << "-dumpstream" << "-dumpfile" << encodeFileOrUrl (rd->record_file);
     qDebug ("mplayer %s\n", args.join (" ").toLocal8Bit ().constData ());
     startProcess (exe, args);
     if (m_process->waitForStarted ()) {
@@ -1360,6 +1402,10 @@ void MasterProcess::pause () {
     }
 }
 
+void MasterProcess::unpause () {
+    pause ();
+}
+
 bool MasterProcess::seek (int pos, bool) {
     if (IProcess::Playing == m_state) {
         MasterProcessInfo *mpi = static_cast<MasterProcessInfo *>(process_info);
@@ -1479,7 +1525,7 @@ KDE_NO_CDTOR_EXPORT ConfigNode::ConfigNode (NodePtr & d, const QString & t)
     : DarkNode (d, t.toUtf8 ()), w (0L) {}
 
 Node *ConfigDocument::childFromTag (const QString & tag) {
-    if (tag.lower () == QString ("document"))
+    if (tag.toLower () == QString ("document"))
         return new ConfigNode (m_doc, tag);
     return 0L;
 }
@@ -1500,8 +1546,8 @@ Node *SomeNode::childFromTag (const QString & t) {
 }
 
 QWidget * TypeNode::createWidget (QWidget * parent) {
-    QString type_attr = getAttribute (Ids::attr_type);
-    const char * ctype = type_attr.ascii ();
+    QByteArray ba = getAttribute (Ids::attr_type).toAscii ();
+    const char *ctype = ba.constData ();
     QString value = getAttribute (Ids::attr_value);
     if (!strcmp (ctype, "range")) {
         w = new QSlider (getAttribute (QString ("START")).toInt (),
@@ -1528,8 +1574,8 @@ QWidget * TypeNode::createWidget (QWidget * parent) {
 
 void TypeNode::changedXML (QTextStream & out) {
     if (!w) return;
-    QString type_attr = getAttribute (Ids::attr_type);
-    const char * ctype = type_attr.ascii ();
+    QByteArray ba = getAttribute (Ids::attr_type).toAscii ();
+    const char *ctype = ba.constData ();
     QString value = getAttribute (Ids::attr_value);
     QString newvalue;
     if (!strcmp (ctype, "range")) {
@@ -1582,13 +1628,10 @@ bool FFMpeg::deMediafiedPlay () {
     if (!rd)
         return false;
     initProcess ();
-    KUrl url (m_url);
     connect (m_process, SIGNAL (finished (int, QProcess::ExitStatus)),
             this, SLOT (processStopped (int, QProcess::ExitStatus)));
-    KUrl out (rd->record_file);
-    QString outurl = QString (QFile::encodeName (out.isLocalFile ()
-                ? getPath (out) : out.url ()));
-    if (out.isLocalFile ())
+    QString outurl = encodeFileOrUrl (rd->record_file);
+    if (outurl.startsWith (QChar ('/')))
         QFile (outurl).remove ();
     QString exe ("ffmpeg ");
     QStringList args;
@@ -1618,10 +1661,10 @@ bool FFMpeg::deMediafiedPlay () {
             process.waitForFinished (5000);
         }
     } else {
-        args << "-i" << QString (QFile::encodeName (url.isLocalFile () ? getPath (url) : url.url ()));
+        args << "-i" << encodeFileOrUrl (m_url);
     }
     args << KShell::splitArgs (m_settings->ffmpegarguments);
-    args << QString (QFile::encodeName (outurl));
+    args << outurl;
     qDebug ("ffmpeg %s\n", args.join (" ").toLocal8Bit().constData ());
     // FIXME if (m_player->source () == source) // ugly
     //    m_player->stop ();
@@ -1678,7 +1721,8 @@ KDE_NO_CDTOR_EXPORT NpStream::NpStream (NpPlayer *p, uint32_t sid, const QString
    job (0L), bytes (0),
    stream_id (sid),
    content_length (0),
-   finish_reason (NoReason) {
+   finish_reason (NoReason),
+   received_data (false) {
     data_arrival.tv_sec = 0;
     (void) new StreamAdaptor (this);
     QString objpath = QString ("%1/stream_%2").arg (p->objectPath ()).arg (sid);
@@ -1708,6 +1752,7 @@ KDE_NO_EXPORT void NpStream::open () {
     } else {
         if (!post.size ()) {
             job = KIO::get (KUrl (url), KIO::NoReload, KIO::HideProgressInfo);
+            job->addMetaData ("PropagateHttpHeader", "true");
         } else {
             QStringList name;
             QStringList value;
@@ -1770,7 +1815,8 @@ KDE_NO_EXPORT void NpStream::close () {
 }
 
 KDE_NO_EXPORT void NpStream::destroy () {
-     static_cast <NpPlayer *> (parent ())->destroyStream (stream_id);
+    pending_buf.clear ();
+    static_cast <NpPlayer *> (parent ())->destroyStream (stream_id);
 }
 
 KDE_NO_EXPORT void NpStream::slotResult (KJob *jb) {
@@ -1781,20 +1827,28 @@ KDE_NO_EXPORT void NpStream::slotResult (KJob *jb) {
 }
 
 KDE_NO_EXPORT void NpStream::slotData (KIO::Job*, const QByteArray& qb) {
-    int sz = pending_buf.size ();
-    if (sz) {
-        pending_buf.resize (sz + qb.size ());
-        memcpy (pending_buf.data () + sz, qb.constData (), qb.size ());
-    } else {
-        pending_buf = qb;
+    if (job) {
+        int sz = pending_buf.size ();
+        if (sz) {
+            pending_buf.resize (sz + qb.size ());
+            memcpy (pending_buf.data () + sz, qb.constData (), qb.size ());
+        } else {
+            pending_buf = qb;
+        }
+        if (sz + qb.size () > 64000 &&
+                !job->isSuspended () && !job->suspend ())
+            kError () << "suspend not supported" << endl;
+        if (!sz)
+            gettimeofday (&data_arrival, 0L);
+        if (!received_data) {
+            received_data = true;
+            http_headers = job->queryMetaData ("HTTP-Headers");
+            if (!http_headers.isEmpty() && !http_headers.endsWith (QChar ('\n')))
+                http_headers += QChar ('\n');
+        }
+        if (sz + qb.size ())
+            emit stateChanged ();
     }
-    if (sz + qb.size () > 64000 &&
-            !job->isSuspended () && !job->suspend ())
-        kError () << "suspend not supported" << endl;
-    if (!sz)
-        gettimeofday (&data_arrival, 0L);
-    if (sz + qb.size ())
-        emit stateChanged ();
 }
 
 KDE_NO_EXPORT void NpStream::redirection (KIO::Job *, const KUrl &kurl) {
@@ -1824,11 +1878,10 @@ KDE_NO_EXPORT void NpPlayer::init () {
 }
 
 KDE_NO_EXPORT void NpPlayer::initProcess () {
-    Process::initProcess ();
+    setupProcess (&m_process);
+    m_process_state = QProcess::NotRunning;
     connect (m_process, SIGNAL (finished (int, QProcess::ExitStatus)),
             this, SLOT (processStopped (int, QProcess::ExitStatus)));
-    connect (m_process, SIGNAL (readyReadStandardOutput ()),
-            this, SLOT (processOutput ()));
     connect (m_process, SIGNAL (readyReadStandardError ()),
             this, SLOT (processOutput ()));
     connect (m_process, SIGNAL (bytesWritten (qint64)),
@@ -1842,21 +1895,62 @@ KDE_NO_EXPORT void NpPlayer::initProcess () {
         filter = QString ("type='method_call',interface='org.kde.kmplayer.callback'");
         service = QDBusConnection::sessionBus().baseService ();
         //service = QString (dbus_bus_get_unique_name (conn));
-        kDebug() << "using service " << service << " interface " << iface << " filter:" << filter.ascii();
+        kDebug() << "using service " << service << " interface " << iface << " filter:" << filter;
     }
 }
 
 KDE_NO_EXPORT bool NpPlayer::deMediafiedPlay () {
-    kDebug() << "NpPlayer::play '" << m_url << "'";
+    kDebug() << "NpPlayer::play '" << m_url << "' state " << m_state;
     // if we change from XPLAIN to XEMBED, the DestroyNotify may come later
     Mrl *node = mrl ();
     if (!view ())
         return false;
-    if (user && user->viewer ()) {
-        user->viewer ()->useIndirectWidget (false);
-        user->viewer ()->setMonitoring (IViewer::MonitorNothing);
+    if (!m_url.isEmpty () && m_url != "about:empty") {
+        QDBusMessage msg = QDBusMessage::createMethodCall (
+                remote_service, "/plugin", "org.kde.kmplayer.backend", "play");
+        msg << m_url;
+        msg.setDelayedReply (false);
+        QDBusConnection::sessionBus().send (msg);
+        setState (IProcess::Buffering);
     }
-    if (node && !m_url.isEmpty ()) {
+    return true;
+}
+
+KDE_NO_EXPORT bool NpPlayer::ready () {
+    Mrl *node = mrl ();
+    if (!node || !user || !user->viewer ())
+        return false;
+
+    user->viewer ()->useIndirectWidget (false);
+    user->viewer ()->setMonitoring (IViewer::MonitorNothing);
+
+    if (state () == IProcess::Ready)
+        return true;
+
+    initProcess ();
+    QString exe ("knpplayer");
+    QStringList args;
+    args << "-cb" << (service + path);
+    args << "-wid" << QString::number (user->viewer ()->windowHandle ());
+    startProcess (exe, args);
+    if (m_process->waitForStarted (5000)) {
+        QString s;
+        for (int i = 0; i < 2 && remote_service.isEmpty (); ++i) {
+            if (!m_process->waitForReadyRead (5000))
+                return false;
+            const QByteArray ba = m_process->readAllStandardOutput ();
+            s += QString::fromAscii (ba.data (), ba.size ());
+            int nl = s.indexOf (QChar ('\n'));
+            if (nl > 0) {
+                int p = s.indexOf ("NPP_DBUS_SRV=");
+                if (p > -1)
+                    remote_service = s.mid (p + 13, nl - p - 13);
+            }
+        }
+    }
+    connect (m_process, SIGNAL (readyReadStandardOutput ()),
+            this, SLOT (processOutput ()));
+    if (!remote_service.isEmpty ()) {
         QString mime = "text/plain";
         QString plugin;
         Element *elm = node;
@@ -1881,37 +1975,22 @@ KDE_NO_EXPORT bool NpPlayer::deMediafiedPlay () {
         }
         if (!plugin.isEmpty ()) {
             QDBusMessage msg = QDBusMessage::createMethodCall (
-                    remote_service, "/plugin", "org.kde.kmplayer.backend", "play");
-            msg << m_url << mime << plugin;
+                    remote_service, "/plugin", "org.kde.kmplayer.backend", "setup");
+            msg << mime << plugin;
             QMap <QString, QVariant> urlargs;
             for (AttributePtr a = elm->attributes ().first (); a; a = a->nextSibling ())
                 urlargs.insert (a->name ().toString (), a->value ());
             msg << urlargs;
             msg.setDelayedReply (false);
-            QDBusConnection::sessionBus().send (msg);
-            setState (IProcess::Buffering);
+            QDBusConnection::sessionBus().call (msg, QDBus::BlockWithGui);
+            setState (IProcess::Ready);
             return true;
         }
     }
+    m_old_state = m_state = Ready;
     stop ();
-    return false;
-}
 
-KDE_NO_EXPORT bool NpPlayer::ready () {
-    if (!user || !user->viewer ())
-        return false;
-    // FIXME wait for callback
-    user->viewer ()->useIndirectWidget (false);
-    if (state () == IProcess::Ready)
-        return true;
-    initProcess ();
-    QString exe ("knpplayer");
-    QStringList args;
-    args << "-cb" << (service + path);
-    args << "-wid" << QString::number (user->viewer ()->windowHandle ());
-    //qDebug ("knpplayer %s", args.join(" ").toLocal8Bit().constData ());
-    startProcess (exe, args);
-    return true;
+    return false;
 }
 
 KDE_NO_EXPORT void NpPlayer::running (const QString &srv) {
@@ -1949,7 +2028,7 @@ KDE_NO_EXPORT void NpPlayer::request_stream (const QString &path, const QString
     }
     kDebug () << "NpPlayer::request " << path << " '" << uri << "'" << m_url << "->" << url;
     Q_UINT32 sid = getStreamId (path);
-    if (sid >= 0) {
+    if ((int)sid >= 0) {
         if (!target.isEmpty ()) {
             kDebug () << "new page request " << target;
             if (js) {
@@ -2041,7 +2120,8 @@ KDE_NO_EXPORT void NpPlayer::quit () {
 }
 
 KDE_NO_EXPORT void NpPlayer::processOutput () {
-    outputToView (view (), m_process->readAllStandardOutput ());
+    if (!remote_service.isEmpty ())
+        outputToView (view (), m_process->readAllStandardOutput ());
     outputToView (view (), m_process->readAllStandardError ());
 }
 
@@ -2072,22 +2152,21 @@ KDE_NO_EXPORT void NpPlayer::streamRedirected (uint32_t sid, const KUrl &u) {
 
 KDE_NO_EXPORT
 void NpPlayer::requestGet (const uint32_t id, const QString &prop, QString *res) {
-    QDBusMessage msg = QDBusMessage::createMethodCall (
-            remote_service, "/plugin", "org.kde.kmplayer.backend", "get");
-    msg << id << prop;
-    QDBusMessage rmsg = QDBusConnection::sessionBus().call (msg, QDBus::BlockWithGui);
-    if (rmsg.type () == QDBusMessage::ReplyMessage) {
-        //kDebug() << "get " << prop << rmsg.arguments ().size ();
-        if (rmsg.arguments ().size ()) {
-            QString s = rmsg.arguments ().first ().toString ();
-            if (s != "error")
-                *res = s;
+    if (!remote_service.isEmpty ()) {
+        QDBusMessage msg = QDBusMessage::createMethodCall (
+                remote_service, "/plugin", "org.kde.kmplayer.backend", "get");
+        msg << id << prop;
+        QDBusMessage rmsg = QDBusConnection::sessionBus().call (msg, QDBus::BlockWithGui);
+        if (rmsg.type () == QDBusMessage::ReplyMessage) {
+            //kDebug() << "get " << prop << rmsg.arguments ().size ();
+            if (rmsg.arguments ().size ()) {
+                QString s = rmsg.arguments ().first ().toString ();
+                if (s != "error")
+                    *res = s;
+            }
         } else {
-            *res = QString ("'null'"); //FIXME
+            kError() << "get" << prop << rmsg.type () << rmsg.errorMessage ();
         }
-    } else {
-        kError() << "get" << prop << rmsg.type () << rmsg.errorMessage ();
-        *res = QString ("'null'"); //FIXME
     }
 }
 
@@ -2139,10 +2218,7 @@ KDE_NO_EXPORT void NpPlayer::processStreams () {
                 (ns->finish_reason == NpStream::BecauseDone &&
                  ns->pending_buf.size () == 0)) {
             sendFinish (i.key(), ns->bytes, ns->finish_reason);
-            StreamMap::iterator ii = i;
-            ++ii;
-            streams.erase (i);
-            i = ii;
+            i = streams.erase (i);
             delete ns;
         } else {
             if (ns->pending_buf.size () > 0 &&
@@ -2158,11 +2234,16 @@ KDE_NO_EXPORT void NpPlayer::processStreams () {
     }
     //kDebug() << "NpPlayer::processStreams " << stream;
     if (stream) {
-        if (!stream->bytes && (!stream->mimetype.isEmpty() || stream->content_length)) {
+        if (stream->finish_reason != NpStream::BecauseStopped &&
+                stream->finish_reason != NpStream::BecauseError &&
+                !stream->bytes &&
+                (!stream->mimetype.isEmpty() || stream->content_length)) {
             QString objpath = QString ("/stream_%1").arg (stream->stream_id);
             QDBusMessage msg = QDBusMessage::createMethodCall (
                     remote_service, objpath, "org.kde.kmplayer.backend", "streamInfo");
-            msg << stream->mimetype << stream->content_length;
+            msg << stream->mimetype
+                << stream->content_length
+                << stream->http_headers;
             msg.setDelayedReply (false);
             QDBusConnection::sessionBus().send (msg);
         }
@@ -2184,16 +2265,29 @@ KDE_NO_EXPORT void NpPlayer::processStreams () {
     in_process_stream = false;
 }
 
-KDE_NO_EXPORT void NpPlayer::wroteStdin (qint64) {
-    write_in_progress = false;
-    if (running ()) {
-        if (in_process_stream) {
-            kDebug() << "sync wroteStdin";
-        }
-        processStreams ();
+KDE_NO_EXPORT void NpPlayer::wroteStdin (qint64 sz) {
+    if (!m_process->bytesToWrite ()) {
+        write_in_progress = false;
+        if (running ())
+            processStreams ();
     }
 }
 
+QString NpPlayer::cookie (const QString &url)
+{
+    QString s;
+    View *v = view ();
+    if (v) {
+        KIO::Integration::CookieJar jar (v);
+        jar.setWindowId (v->topLevelWidget()->winId ());
+        QList<QNetworkCookie> c = jar.cookiesForUrl (url);
+        QList<QNetworkCookie>::const_iterator e = c.end ();
+        for (QList<QNetworkCookie>::const_iterator i = c.begin (); i != e; ++i)
+            s += (s.isEmpty() ? "" : ";") + QString::fromUtf8 ((*i).toRawForm());
+    }
+    return s;
+}
+
 #else
 
 KDE_NO_CDTOR_EXPORT NpStream::NpStream (NpPlayer *p, uint32_t sid, const QString &u, const QByteArray &/*ps*/)
