@@ -76,13 +76,14 @@ static NPNetscapeFuncs ns_funcs;
 static NPPluginFuncs np_funcs;       /* plugin functions              */
 static NPP npp;                      /* single instance of the plugin */
 static NPWindow np_window;
+static NPSetWindowCallbackStruct ws_info;
 static NPObject *js_window;
 static NPSavedData *saved_data;
 static NPClass js_class;
 static GTree *stream_list;
 static gpointer current_stream_id;
 static uint32_t stream_chunk_size;
-static char stream_buf[32 * 1024];
+static char stream_buf[64 * 1024];
 static unsigned int stream_buf_pos;
 static int stream_id_counter;
 static GTree *identifiers;
@@ -97,6 +98,7 @@ typedef struct _StreamInfo {
     char *mimetype;
     char *target;
     char *post;
+    char *headers;
     bool notify;
     bool called_plugin;
     bool destroyed;
@@ -112,7 +114,9 @@ static NP_GetValueUPP npGetValue;
 static NP_InitializeUPP npInitialize;
 static NP_ShutdownUPP npShutdown;
 
-static void callFunction(int stream, const char *func, int first_arg_type, ...);
+static const char *iface_stream = "org.kde.kmplayer.stream";
+static const char *iface_callback = "org.kde.kmplayer.callback";
+static void callFunction(int stream, const char *iface, const char *func, int first_arg_type, ...);
 static void readStdin (gpointer d, gint src, GdkInputCondition cond);
 static char *evaluate (const char *script, bool store);
 
@@ -124,10 +128,11 @@ static void dbusStreamUnregister (DBusConnection *conn, void *user_data);
 
 static void print (const char * format, ...) {
     va_list vl;
+    static FILE *out = stderr; //fopen ("/tmp/npp.out", "w+");
     va_start (vl, format);
-    vprintf (format, vl);
+    vfprintf (out, format, vl);
     va_end (vl);
-    fflush (stdout);
+    fflush (out);
 }
 
 static void *nsAlloc (uint32 size) {
@@ -155,7 +160,7 @@ static gint streamCompare (gconstpointer a, gconstpointer b) {
 
 static void freeStream (StreamInfo *si) {
     char stream_name[64];
-    sprintf (stream_name, "/stream_%d", (long) si->np_stream.ndata);
+    sprintf (stream_name, "/stream_%d", (int)(long) si->np_stream.ndata);
     if (!g_tree_remove (stream_list, si->np_stream.ndata))
         print ("WARNING freeStream not in tree\n");
     else
@@ -167,6 +172,8 @@ static void freeStream (StreamInfo *si) {
         g_free (si->target);
     if (si->post)
         nsMemFree (si->post);
+    if (si->headers)
+        nsMemFree (si->headers);
     nsMemFree (si);
 }
 
@@ -217,9 +224,10 @@ static gboolean requestStream (void * p) {
 
 static gboolean destroyStream (void * p) {
     StreamInfo *si = (StreamInfo *) g_tree_lookup (stream_list, p);
-    print ("FIXME destroyStream\n");
-    if (si)
-        callFunction ((int)(long)p, "destroy", DBUS_TYPE_INVALID);
+    if (si) {
+        si->destroyed = true;
+        callFunction ((int)(long)p, iface_stream, "destroy", DBUS_TYPE_INVALID);
+    }
     return 0; /* single shot */
 }
 
@@ -246,7 +254,7 @@ static int32_t writeStream (gpointer p, char *buf, uint32_t count) {
     StreamInfo *si = (StreamInfo *) g_tree_lookup (stream_list, p);
     /*print ("writeStream found %d count %d\n", !!si, count);*/
     if (si) {
-        if (si->reason > NPERR_NO_ERROR) {
+        if (si->reason > NPERR_NO_ERROR || si->destroyed) {
             sz = count; /* stream closed, skip remainings */
         } else {
             if (!si->called_plugin) {
@@ -271,7 +279,7 @@ static int32_t writeStream (gpointer p, char *buf, uint32_t count) {
                 if (sz < 0) { /*FIXME plugin destroys stream here*/
                     si->reason = NPERR_INVALID_PLUGIN_ERROR;
                     destroyStream ((gpointer)p);
-                    return 0;
+                    return count; /* stream not accepted, skip remainings */
                 }
             } else {
                 sz = 0;
@@ -310,7 +318,7 @@ static StreamInfo *addStream (const char *url, const char *mime, const char *tar
     si->notify = notify;
     si->np_stream.ndata = (void *) (long) (stream_id_counter++);
     print ("add stream %d\n", (long) si->np_stream.ndata);
-    sprintf (stream_name, "/stream_%d", (long) si->np_stream.ndata);
+    sprintf (stream_name, "/stream_%d", (int)(long) si->np_stream.ndata);
     if (!dbus_connection_register_object_path (dbus_connection, stream_name,
                 &stream_vtable, si))
         g_printerr ("dbus_connection_register_object_path error\n");
@@ -550,6 +558,12 @@ static NPError nsGetValue (NPP instance, NPNVariable variable, void *value) {
         case NPNVSupportsXEmbedBool:
             *(NPBool*)value = 1;
             break;
+        case NPNVSupportsWindowless:
+            *(NPBool*)value = 0;
+            break;
+        case NPNVprivateModeBool:
+            *(NPBool*)value = 0;
+            break;
         case NPNVWindowNPObject:
             if (!js_window) {
                 JsObject *jo = (JsObject*) nsCreateObject (instance, &js_class);
@@ -565,7 +579,6 @@ static NPError nsGetValue (NPP instance, NPNVariable variable, void *value) {
             break;
         }
         default:
-            *(int*)value = 0;
             print ("unknown value\n");
             return NPERR_GENERIC_ERROR;
     }
@@ -829,10 +842,38 @@ static bool nsConstruct (NPP npp, NPObject* obj, const NPVariant *args, uint32_t
 }
 
 static NPError nsGetValueForURL (NPP npp, NPNURLVariable variable, const char *url, char **value, uint32_t *len) {
-    (void)npp; (void)url; (void)value;
+    (void)npp;
     print ("NPN_GetValueForURL\n");
     switch (variable) {
     case NPNURLVCookie:
+        if (callback_service) {
+            DBusMessage *rmsg;
+            DBusMessage *msg = dbus_message_new_method_call (
+                    callback_service,
+                    callback_path,
+                    "org.kde.kmplayer.callback",
+                    "cookie");
+            dbus_message_append_args (msg,
+                    DBUS_TYPE_STRING, &url,
+                    DBUS_TYPE_INVALID);
+            rmsg = dbus_connection_send_with_reply_and_block (dbus_connection,
+                    msg, 2000, NULL);
+            if (rmsg) {
+                DBusMessageIter it;
+                if (dbus_message_iter_init (rmsg, &it) &&
+                      DBUS_TYPE_STRING == dbus_message_iter_get_arg_type (&it)) {
+                    char *cookies;
+                    dbus_message_iter_get_basic (&it, &cookies);
+                    *value = g_strdup (cookies);
+                    *len = strlen (cookies);
+                    print ("cookie %s => %s\n", url, cookies);
+                }
+                dbus_message_unref (rmsg);
+                break;
+            } else {
+                print ("cookie no reply\n");
+            }
+        }
     case NPNURLVProxy:
     default:
         *value = (char *) nsAlloc (1);
@@ -1004,7 +1045,8 @@ static bool windowClassInvoke (NPObject *npobj, NPIdentifier method,
     nsMemFree ((char *) str.utf8characters);
     for (i = 0; i < arg_count; i++) {
         char *arg = nsVariant2Str (args + i);
-        pos += snprintf (buf + pos, sizeof (buf) - pos, i ? ",%s" : "%s", arg);
+        pos += snprintf (buf + pos,
+                sizeof (buf) - pos, i ? ",%s" : "%s", *arg ? arg : "undefined");
         nsMemFree (arg);
     }
     pos += snprintf (buf + pos, sizeof (buf) - pos, ")");
@@ -1184,7 +1226,7 @@ static int initPlugin (const char *plugin_lib) {
     NPError np_err;
     char *pname;
 
-    print ("starting %s with %s\n", plugin_lib, object_url);
+    print ("starting %s\n", plugin_lib);
     library = g_module_open (plugin_lib, G_MODULE_BIND_LAZY);
     if (!library) {
         print ("failed to load %s %s\n", plugin_lib, g_module_error ());
@@ -1292,7 +1334,6 @@ static int initPlugin (const char *plugin_lib) {
 }
 
 static int newPlugin (NPMIMEType mime, int16 argc, char *argn[], char *argv[]) {
-    NPSetWindowCallbackStruct ws_info;
     NPError np_err;
     Display *display;
     int screen;
@@ -1364,13 +1405,11 @@ static int newPlugin (NPMIMEType mime, int16 argc, char *argn[], char *argv[]) {
     return 0;
 }
 
-static gpointer startPlugin (const char *url, const char *mime,
+static bool startPlugin (const char *mime,
         int argc, char *argn[], char *argv[]) {
-    StreamInfo *si;
     if (!npp && (initPlugin (plugin) || newPlugin (mimetype, argc, argn, argv)))
-        return 0L;
-    si = addStream (url, mime, 0L, 0, NULL, 0L, false);
-    return si;
+        return false;
+    return true;
 }
 
 /*----------------%<---------------------------------------------------------*/
@@ -1448,6 +1487,7 @@ static DBusHandlerResult dbusStreamMessage (DBusConnection *conn,
     } else if (dbus_message_is_method_call (msg, iface, "streamInfo")) {
         const char *mime;
         if (dbusMsgIterGet (msg, &args, DBUS_TYPE_STRING, &mime, true)) {
+            const char *headers;
             uint32_t length;
             si = getStreamInfo(dbus_message_get_path (msg), &stream_id);
             if (si && *mime) {
@@ -1458,9 +1498,15 @@ static DBusHandlerResult dbusStreamMessage (DBusConnection *conn,
             if (dbusMsgIterGet (msg, &args, DBUS_TYPE_UINT32, &length, false)) {
                 if (si)
                     si->np_stream.end = length;
+                if (dbusMsgIterGet (msg, &args, DBUS_TYPE_STRING, &headers, false)) {
+                    if (si && *headers) {
+                        si->headers = g_strdup (headers);
+                        si->np_stream.headers = si->headers;
+                    }
+                }
             }
-            print ("streamInfo %d size:%d mime:%s\n", (long)stream_id, length,
-                    mime ? mime : "");
+            print ("streamInfo %d size:%d mime:%s headers %s\n", (long)stream_id,
+                    length, mime ? mime : "", headers ? headers : "");
         }
         defaultReply (conn, msg);
     }
@@ -1487,12 +1533,14 @@ static const char *plugin_inspect =
     "  </method>"
     " </interface>"
     "  <interface name=\"org.kde.kmplayer.backend\">"
-    "   <method name=\"play\">"
-    "    <arg name=\"url\" type=\"s\" direction=\"in\"/>"
+    "   <method name=\"setup\">"
     "    <arg name=\"mimetype\" type=\"s\" direction=\"in\"/>"
     "    <arg name=\"plugin\" type=\"s\" direction=\"in\"/>"
     "    <arg name=\"arguments\" type=\"a{sv}\" direction=\"in\"/>"
     "   </method>"
+    "   <method name=\"play\">"
+    "    <arg name=\"url\" type=\"s\" direction=\"in\"/>"
+    "   </method>"
     "  </interface>"
     "</node>";
 
@@ -1513,7 +1561,7 @@ static DBusHandlerResult dbusPluginMessage (DBusConnection *conn,
         dbus_connection_send (conn, rmsg, NULL);
         dbus_connection_flush (conn);
         dbus_message_unref (rmsg);
-    } else if (dbus_message_is_method_call (msg, iface, "play")) {
+    } else if (dbus_message_is_method_call (msg, iface, "setup")) {
         DBusMessageIter ait;
         char *param = 0;
         unsigned int params = 0;
@@ -1521,11 +1569,6 @@ static DBusHandlerResult dbusPluginMessage (DBusConnection *conn,
         char **argv = NULL;
         GSList *arglst = NULL;
         if (!dbusMsgIterGet (msg, &args, DBUS_TYPE_STRING, &param, true)) {
-            g_printerr ("missing url arg");
-            return DBUS_HANDLER_RESULT_HANDLED;
-        }
-        object_url = g_strdup (param);
-        if (!dbusMsgIterGet (msg, &args, DBUS_TYPE_STRING, &param, false)) {
             g_printerr ("missing mimetype arg");
             return DBUS_HANDLER_RESULT_HANDLED;
         }
@@ -1584,9 +1627,20 @@ static DBusHandlerResult dbusPluginMessage (DBusConnection *conn,
             }
             g_slist_free (arglst);
         }
-        print ("play %s %s %s params:%d\n", object_url,
+        print ("setup %s %s params:%d\n",
                 mimetype ? mimetype : "", plugin, params);
-        startPlugin (object_url, mimetype, params, argn, argv);
+        startPlugin (mimetype, params, argn, argv);
+        defaultReply (conn, msg);
+    } else if (dbus_message_is_method_call (msg, iface, "play")) {
+        char *param = 0;
+        if (!dbusMsgIterGet (msg, &args, DBUS_TYPE_STRING, &param, true)) {
+            g_printerr ("missing url arg");
+            return DBUS_HANDLER_RESULT_HANDLED;
+        }
+        object_url = g_strdup (param);
+        print ("play %s\n", object_url);
+        if (mimetype && strncmp (mimetype, "application/x-java", 18))
+            addStream (object_url, mimetype, 0L, 0, NULL, 0L, false);
         defaultReply (conn, msg);
     } else if (dbus_message_is_method_call (msg, iface, "get")) {
         DBusMessage * rmsg;
@@ -1657,7 +1711,8 @@ static DBusHandlerResult dbusPluginMessage (DBusConnection *conn,
     return DBUS_HANDLER_RESULT_HANDLED;
 }
 
-static void callFunction(int stream,const char *func, int first_arg_type, ...) {
+static void callFunction(int stream, const char *iface, const char *func, int first_arg_type, ...)
+{
     char path[64];
     createPath (stream, path, sizeof (path));
     print ("call %s.%s()\n", path, func);
@@ -1666,7 +1721,7 @@ static void callFunction(int stream,const char *func, int first_arg_type, ...) {
         DBusMessage *msg = dbus_message_new_method_call (
                 callback_service,
                 path,
-                "org.kde.kmplayer.callback",
+                iface,
                 func);
         if (first_arg_type != DBUS_TYPE_INVALID) {
             va_start (var_args, first_arg_type);
@@ -1745,7 +1800,7 @@ static void pluginAdded (GtkSocket *socket, gpointer d) {
                     );
         }
     }
-    callFunction (-1, "plugged", DBUS_TYPE_INVALID);
+    callFunction (-1, iface_callback, "plugged", DBUS_TYPE_INVALID);
 }
 
 static void windowCreatedEvent (GtkWidget *w, gpointer d) {
@@ -1770,7 +1825,8 @@ static void windowCreatedEvent (GtkWidget *w, gpointer d) {
     if (!callback_service) {
         char *argn[] = { "WIDTH", "HEIGHT", "debug", "SRC" };
         char *argv[] = { "440", "330", g_strdup("yes"), g_strdup(object_url) };
-        startPlugin (object_url, mimetype, 4, argn, argv);
+        if (startPlugin (mimetype, 4, argn, argv))
+            addStream (object_url, mimetype, 0L, 0, NULL, 0L, false);
     }
 }
 
@@ -1892,10 +1948,13 @@ static gboolean initPlayer (void * p) {
 
 
         /* TODO: remove DBUS_BUS_SESSION and create a private connection */
-        callFunction (-1, "running",
-                DBUS_TYPE_STRING, &service_name, DBUS_TYPE_INVALID);
+        //callFunction (-1, "running",
+        //        DBUS_TYPE_STRING, &service_name, DBUS_TYPE_INVALID);
 
         dbus_connection_flush (dbus_connection);
+
+        fprintf (stdout, "NPP_DBUS_SRV=%s\n", service_name);
+        fflush (stdout);
     }
     return 0; /* single shot */
 }
