@@ -25,7 +25,6 @@
 #include <qapplication.h>
 #include <qslider.h>
 #include <qcursor.h>
-#include <qimage.h>
 #include <qmap.h>
 #include <QPalette>
 #include <QDesktopWidget>
@@ -33,6 +32,10 @@
 #include <QPainter>
 #include <QMainWindow>
 #include <QWidgetAction>
+#include <QtGui/QTextBlock>
+#include <QtGui/QTextDocument>
+#include <QtGui/QAbstractTextDocumentLayout>
+#include <QtGui/QImage>
 
 #include <kactioncollection.h>
 #include <kapplication.h>
@@ -49,7 +52,6 @@
 #ifdef KMPLAYER_WITH_CAIRO
 # include <cairo-xlib.h>
 # include <cairo-xlib-xrender.h>
-# include <pango/pangocairo.h>
 #endif
 #include "mediaobject.h"
 #include "kmplayer_smil.h"
@@ -94,7 +96,7 @@ void ImageData::copyImage (Surface *s, const SSize &sz, cairo_surface_t *similar
         src_sf = surface;
     } else {
         if (image->depth () < 24) {
-            QImage qi = image->convertDepth (32, 0);
+            QImage qi = image->convertToFormat (QImage::Format_RGB32);
             *image = qi;
         }
         src_sf = cairo_image_surface_create_for_data (
@@ -202,7 +204,6 @@ public:
     void visit (SMIL::Layout *);
     void visit (SMIL::RegionBase *);
     void visit (SMIL::Transition *);
-    void visit (SMIL::ImageMediaType *);
     void visit (SMIL::TextMediaType *);
     void visit (SMIL::Brush *);
     void visit (SMIL::SmilText *);
@@ -635,14 +636,15 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Transition *trans) {
 KDE_NO_EXPORT void CairoPaintVisitor::video (Mrl *m, Surface *s) {
     if (m->media_info &&
             m->media_info->media &&
-            MediaManager::AudioVideo == m->media_info->type) {
+            (MediaManager::Audio == m->media_info->type ||
+             MediaManager::AudioVideo == m->media_info->type)) {
         AudioVideoMedia *avm = static_cast<AudioVideoMedia *> (m->media_info->media);
         if (avm->viewer ()) {
             if (s &&
                     avm->process &&
                     avm->process->state () > IProcess::Ready &&
                     strcmp (m->nodeName (), "audio")) {
-                s->xscale = s->yscale = 1; // either scale width/heigt or use bounds
+                s->xscale = s->yscale = 1; // either scale width/height or use bounds
                 avm->viewer ()->setGeometry (s->toScreen (s->bounds.size));
             } else {
                 avm->viewer ()->setGeometry (IRect (-60, -60, 50, 50));
@@ -653,10 +655,37 @@ KDE_NO_EXPORT void CairoPaintVisitor::video (Mrl *m, Surface *s) {
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RefMediaType *ref) {
     Surface *s = ref->surface ();
-    if (s && ref->external_tree)
+    if (s && ref->external_tree) {
         updateExternal (ref, s);
-    else
+        return;
+    }
+    if (!ref->media_info)
+        return;
+    if (ref->media_info->media &&
+            ref->media_info->media->type () == MediaManager::Image) {
+        if (!s)
+            return;
+
+        IRect scr = matrix.toScreen (s->bounds);
+        IRect clip_rect = clip.intersect (scr);
+        if (clip_rect.isEmpty ())
+            return;
+
+        ImageMedia *im = static_cast <ImageMedia *> (ref->media_info->media);
+        ImageData *id = im ? im->cached_img.ptr () : NULL;
+        if (id && id->flags == ImageData::ImageScalable)
+            im->render (scr.size);
+        if (!id || im->isEmpty () || ref->size.isEmpty ()) {
+            s->remove();
+            return;
+        }
+        if (!s->surface || s->dirty)
+            id->copyImage (s, SSize (scr.width (), scr.height ()), cairo_surface, ref->pan_zoom);
+        paint (&ref->transition, ref->media_opacity, s, scr.point, clip_rect);
+        s->dirty = false;
+    } else {
         video (ref, s);
+    }
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::paint (TransitionModule *trans,
@@ -708,7 +737,8 @@ static Mrl *findActiveMrl (Node *n, bool *rp_or_smil) {
              mrl->id < RP::id_node_last);
         if (*rp_or_smil ||
                 (mrl->media_info &&
-                 MediaManager::AudioVideo == mrl->media_info->type))
+                 (MediaManager::Audio == mrl->media_info->type ||
+                  MediaManager::AudioVideo == mrl->media_info->type)))
             return mrl;
     }
     for (Node *c = n->firstChild (); c; c = c->nextSibling ())
@@ -752,61 +782,35 @@ void CairoPaintVisitor::updateExternal (SMIL::MediaType *av, SurfacePtr s) {
     paint (&av->transition, av->media_opacity, s.ptr (), scr.point, clip_rect);
 }
 
-KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
-    if (!img->media_info)
-        return;
-    Surface *s = img->surface ();
-    if (!s)
-        return;
-    if (img->external_tree) {
-        updateExternal (img, s);
-        return;
-    }
-    if (!img->media_info->media)
-        return;
-
-    IRect scr = matrix.toScreen (s->bounds);
-    IRect clip_rect = clip.intersect (scr);
-    if (clip_rect.isEmpty ())
-        return;
-
-    ImageMedia *im = static_cast <ImageMedia *> (img->media_info->media);
-    ImageData *id = im ? im->cached_img.ptr () : NULL;
-    if (id && id->flags == ImageData::ImageScalable)
-        im->render (scr.size);
-    if (!id || im->isEmpty () || img->size.isEmpty ()) {
-        s->remove();
-        return;
-    }
-    if (!s->surface || s->dirty)
-        id->copyImage (s, SSize (scr.width (), scr.height ()), cairo_surface, img->pan_zoom);
-    paint (&img->transition, img->media_opacity, s, scr.point, clip_rect);
-    s->dirty = false;
+static void setAlignment (QTextDocument &td, unsigned char align) {
+    QTextOption opt = td.defaultTextOption();
+    if (SmilTextProperties::AlignLeft == align)
+        opt.setAlignment (Qt::AlignLeft);
+    else if (SmilTextProperties::AlignCenter == align)
+        opt.setAlignment (Qt::AlignCenter);
+    else if (SmilTextProperties::AlignRight == align)
+        opt.setAlignment (Qt::AlignRight);
+    td.setDefaultTextOption (opt);
 }
 
-static void calculateTextDimensions (PangoFontDescription *desc,
-        const char *text, Single w, Single h,
-        int *pxw, int *pxh, bool markup_text, int align = -1) {
-    cairo_surface_t *img_surf = cairo_image_surface_create (
-            CAIRO_FORMAT_RGB24, (int) w, h);
-    cairo_t *cr_txt = cairo_create (img_surf);
-    PangoLayout *layout = pango_cairo_create_layout (cr_txt);
-    pango_layout_set_width (layout, 1.0 * w * PANGO_SCALE);
-    pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
+static void calculateTextDimensions (const QFont& font,
+        const QString& text, Single w, Single h, Single maxh,
+        int *pxw, int *pxh, bool markup_text,
+        unsigned char align = SmilTextProperties::AlignLeft) {
+    QTextDocument td;
+    td.setDefaultFont( font );
+    td.setDocumentMargin (0);
+    QImage img (QSize ((int)w, (int)h), QImage::Format_RGB32);
+    td.setPageSize (QSize ((int)w, (int)maxh));
+    td.documentLayout()->setPaintDevice (&img);
     if (markup_text)
-        pango_layout_set_markup (layout, text, -1);
+        td.setHtml( text );
     else
-        pango_layout_set_text (layout, text, -1);
-    if (align > -1)
-        pango_layout_set_alignment (layout, (PangoAlignment) align);
-    pango_layout_set_font_description (layout, desc);
-
-    pango_cairo_show_layout (cr_txt, layout);
-    pango_layout_get_pixel_size (layout, pxw, pxh);
-
-    g_object_unref (layout);
-    cairo_destroy (cr_txt);
-    cairo_surface_destroy (img_surf);
+        td.setPlainText( text );
+    setAlignment (td, align);
+    QRectF r = td.documentLayout()->blockBoundingRect (td.lastBlock());
+    *pxw = (int)td.idealWidth ();
+    *pxh = (int)(r.y() + r.height());
 }
 
 static cairo_t *createContext (cairo_surface_t *similar, Surface *s, int w, int h) {
@@ -850,32 +854,46 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
         int w = scr.width ();
         int pxw, pxh;
         Single ft_size = w * txt->font_size / (double)s->bounds.width ();
-        const QByteArray text = tm->text.toUtf8 ();
         bool clear = s->surface;
 
-        PangoFontDescription *desc = pango_font_description_new ();
-        pango_font_description_set_family(desc, txt->font_name.toUtf8().data());
-        pango_font_description_set_size (desc, PANGO_SCALE * ft_size);
+        QFont font (txt->font_name, ft_size);
         if (clear) {
             pxw = scr.width ();
             pxh = scr.height ();
         } else {
-            calculateTextDimensions (desc, text.data (),
-                    w, 2 * ft_size, &pxw, &pxh, false);
+            calculateTextDimensions (font, tm->text,
+                    w, 2 * ft_size, scr.height (), &pxw, &pxh, false);
         }
+        QTextDocument td;
+        td.setDocumentMargin (0);
+        td.setDefaultFont (font);
+        bool have_alpha = (s->background_color & 0xff000000) < 0xff000000;
+        QImage img (QSize (pxw, pxh), have_alpha ? QImage::Format_ARGB32 : QImage::Format_RGB32);
+        img.fill (s->background_color);
+        td.setPageSize (QSize (pxw, pxh + (int)ft_size));
+        td.documentLayout()->setPaintDevice (&img);
+        setAlignment (td, 1 + (int)txt->halign);
+        td.setPlainText (tm->text);
+        QPainter painter;
+        painter.begin (&img);
+        QAbstractTextDocumentLayout::PaintContext ctx;
+        ctx.clip = QRect (0, 0, pxw, pxh);
+        ctx.palette.setColor (QPalette::Text, QColor (QRgb (txt->font_color)));
+        td.documentLayout()->draw (&painter, ctx);
+        painter.end();
+
         cairo_t *cr_txt = createContext (cairo_surface, s, pxw, pxh);
-
-        CAIRO_SET_SOURCE_RGB (cr_txt, txt->font_color);
-        PangoLayout *layout = pango_cairo_create_layout (cr_txt);
-        pango_layout_set_width (layout, 1.0 * w * PANGO_SCALE);
-        pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
-        pango_layout_set_text (layout, text.data (), -1);
-        pango_layout_set_font_description (layout, desc);
-
-        pango_cairo_show_layout (cr_txt, layout);
-
-        pango_font_description_free (desc);
-        g_object_unref (layout);
+        cairo_surface_t *src_sf = cairo_image_surface_create_for_data (
+                img.bits (),
+                have_alpha ? CAIRO_FORMAT_ARGB32:CAIRO_FORMAT_RGB24,
+                img.width(), img.height(), img.bytesPerLine ());
+        cairo_pattern_t *pat = cairo_pattern_create_for_surface (src_sf);
+        cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
+        cairo_set_operator (cr_txt, CAIRO_OPERATOR_SOURCE);
+        cairo_set_source (cr_txt, pat);
+        cairo_paint (cr_txt);
+        cairo_pattern_destroy (pat);
+        cairo_surface_destroy (src_sf);
         cairo_destroy (cr_txt);
 
         // update bounds rect
@@ -908,16 +926,23 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Brush * brush) {
             cairo_rectangle (cr, clip_rect.x (), clip_rect.y (),
                     clip_rect.width (), clip_rect.height ());
         }
+        unsigned int color = brush->color.color;
+        if (!color) {
+            color = brush->background_color.color;
+            opacity *= brush->background_color.opacity / 100.0;
+        } else {
+            opacity *= brush->color.opacity / 100.0;
+        }
         opacity *= brush->media_opacity.opacity / 100.0;
         if (opacity < 0.99) {
             cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
             cairo_set_source_rgba (cr,
-                    1.0 * ((brush->color >> 16) & 0xff) / 255,
-                    1.0 * ((brush->color >> 8) & 0xff) / 255,
-                    1.0 * (brush->color & 0xff) / 255,
+                    1.0 * ((color >> 16) & 0xff) / 255,
+                    1.0 * ((color >> 8) & 0xff) / 255,
+                    1.0 * (color & 0xff) / 255,
                     opacity);
         } else {
-            CAIRO_SET_SOURCE_RGB (cr, brush->color);
+            CAIRO_SET_SOURCE_RGB (cr, color);
         }
         cairo_fill (cr);
         if (opacity < 0.99)
@@ -928,17 +953,14 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Brush * brush) {
 }
 
 struct SmilTextBlock {
-    SmilTextBlock (PangoFontDescription *d, const QString &t, IRect r, int a)
-        : desc (d), rich_text (t), rect (r), align (a), next (NULL) {}
+    SmilTextBlock (const QFont& f, const QString &t,
+            IRect r, unsigned char a)
+        : font (f), rich_text (t), rect (r), align (a), next (NULL) {}
 
-    ~SmilTextBlock () {
-        pango_font_description_free (desc);
-    }
-
-    PangoFontDescription *desc;
+    QFont font;
     QString rich_text;
     IRect rect;
-    int align;
+    unsigned char align;
 
     SmilTextBlock *next;
 };
@@ -962,6 +984,7 @@ public:
     using Visitor::visit;
     void visit (TextNode *);
     void visit (SMIL::TextFlow *);
+    void visit (SMIL::TemporalMoment *);
 
     void addRichText (const QString &txt);
     void push ();
@@ -978,40 +1001,40 @@ public:
 };
 
 void SmilTextInfo::span (float scale) {
-    QString s = "<span";
-    if (props.font_size > -1)
-        s += " size='" + QString::number ((int)(1024 * scale * props.font_size)) + "'";
-    s += " face='" + props.font_family + "'";
+    QString s = "<span style=\"";
+    if (props.font_size.size () > -1)
+        s += "font-size:" + QString::number ((int)(scale * props.font_size.size ())) + "px;";
+    s += "font-family:" + props.font_family + ";";
     if (props.font_color > -1)
-        s += QString().sprintf (" foreground='#%06x'", props.font_color);
+        s += QString().sprintf ("color:#%06x;", props.font_color);
     if (props.background_color > -1)
-        s += QString().sprintf (" background='#%06x'", props.background_color);
+        s += QString().sprintf ("background-color:#%06x;", props.background_color);
     if (SmilTextProperties::StyleInherit != props.font_style) {
-        s += " style='";
+        s += "font-style:";
         switch (props.font_style) {
             case SmilTextProperties::StyleOblique:
-                s += "oblique'";
+                s += "oblique;";
                 break;
             case SmilTextProperties::StyleItalic:
-                s += "italic'";
+                s += "italic;";
                 break;
             default:
-                s += "normal'";
+                s += "normal;";
                 break;
         }
     }
     if (SmilTextProperties::WeightInherit != props.font_weight) {
-        s += " weight='";
+        s += "font-weight:";
         switch (props.font_weight) {
             case SmilTextProperties::WeightBold:
-                s += "bold'";
+                s += "bold;";
                 break;
             default:
-                s += "normal'";
+                s += "normal;";
                 break;
         }
     }
-    s += ">";
+    s += "\">";
     span_text = s;
 }
 
@@ -1026,7 +1049,7 @@ void SmilTextVisitor::addRichText (const QString &txt) {
 void SmilTextVisitor::push () {
     if (!rich_text.isEmpty ()) {
         int pxw, pxh;
-        float fs = info.props.font_size;
+        float fs = info.props.font_size.size ();
         if (fs < 0)
             fs = TextMedia::defaultFontSize ();
         float maxfs = max_font_size;
@@ -1035,25 +1058,16 @@ void SmilTextVisitor::push () {
         fs *= scale;
         maxfs *= scale;
 
-        int align = -1;
-        if (SmilTextProperties::AlignLeft == info.props.text_align)
-            align = (int) PANGO_ALIGN_LEFT;
-        else if (SmilTextProperties::AlignCenter == info.props.text_align)
-            align = (int) PANGO_ALIGN_CENTER;
-        else if (SmilTextProperties::AlignRight == info.props.text_align)
-            align = (int) PANGO_ALIGN_RIGHT;
-        PangoFontDescription *desc = pango_font_description_new ();
-        pango_font_description_set_family (desc, "Sans");
-        pango_font_description_set_size (desc, PANGO_SCALE * fs);
-        calculateTextDimensions (desc, rich_text.toUtf8 ().data (),
-                width, 2 * maxfs, &pxw, &pxh, true, align);
+        QFont font ("Sans", (int)fs);
+        calculateTextDimensions (font, rich_text.toUtf8 ().constData (),
+                width, 2 * maxfs, 1024, &pxw, &pxh, true, info.props.text_align);
         int x = 0;
         if (SmilTextProperties::AlignCenter == info.props.text_align)
             x = (width - pxw) / 2;
         else if (SmilTextProperties::AlignRight == info.props.text_align)
             x = width - pxw;
-        SmilTextBlock *block = new SmilTextBlock (desc, rich_text,
-                IRect (x, voffset, pxw, pxh), align);
+        SmilTextBlock *block = new SmilTextBlock (font, rich_text,
+                IRect (x, voffset, pxw, pxh), info.props.text_align);
         voffset += pxh;
         max_font_size = 0;
         rich_text.clear();
@@ -1068,7 +1082,7 @@ void SmilTextVisitor::push () {
 
 void SmilTextVisitor::visit (TextNode *text) {
     QString buffer;
-    QTextStream out (&buffer);
+    QTextStream out (&buffer, QIODevice::WriteOnly);
     out << XMLStringlet (text->nodeValue ());
     addRichText (buffer);
     if (text->nextSibling ())
@@ -1076,11 +1090,11 @@ void SmilTextVisitor::visit (TextNode *text) {
 }
 
 void SmilTextVisitor::visit (SMIL::TextFlow *flow) {
-    if (flow->firstChild ()) {
-        bool new_block = SMIL::id_node_p == flow->id ||
-            SMIL::id_node_br == flow->id ||
-            SMIL::id_node_div == flow->id;
-        float fs = info.props.font_size;
+    bool new_block = SMIL::id_node_p == flow->id ||
+        SMIL::id_node_br == flow->id ||
+        SMIL::id_node_div == flow->id;
+    if ((new_block && !rich_text.isEmpty ()) || flow->firstChild ()) {
+        float fs = info.props.font_size.size ();
         if (fs < 0)
             fs = TextMedia::defaultFontSize ();
         int par_extra = SMIL::id_node_p == flow->id
@@ -1092,15 +1106,16 @@ void SmilTextVisitor::visit (SMIL::TextFlow *flow) {
             push ();
 
         info.props.mask (flow->props);
-        if (info.props.font_size > max_font_size)
-            max_font_size = info.props.font_size;
+        if ((float)info.props.font_size.size () > max_font_size)
+            max_font_size = info.props.font_size.size ();
         info.span (scale);
 
-        flow->firstChild ()->accept (this);
+        if (flow->firstChild ())
+            flow->firstChild ()->accept (this);
 
         if (rich_text.isEmpty ())
             par_extra = 0;
-        if (new_block)
+        if (new_block && flow->firstChild ())
             push ();
         voffset += par_extra;
 
@@ -1110,6 +1125,12 @@ void SmilTextVisitor::visit (SMIL::TextFlow *flow) {
         flow->nextSibling ()->accept (this);
 }
 
+void SmilTextVisitor::visit (SMIL::TemporalMoment *tm) {
+    if (tm->state >= Node::state_began
+            && tm->nextSibling ())
+        tm->nextSibling ()->accept (this);
+}
+
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
     Surface *s = txt->surface ();
     if (!s)
@@ -1124,8 +1145,16 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
         float scale = 1.0 * w / (double)s->bounds.width ();
         SmilTextVisitor info (w, scale, txt->props);
 
-        if (txt->firstChild ())
-            txt->firstChild ()->accept (&info);
+        Node *first = txt->firstChild ();
+        for (Node *n = first; n; n = n->nextSibling ())
+            if (SMIL::id_node_clear == n->id) {
+                if (n->state >= Node::state_began)
+                    first = n->nextSibling ();
+                else
+                    break;
+            }
+        if (first)
+            first->accept (&info);
 
         info.push ();
         if (info.first) {
@@ -1133,23 +1162,42 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
 
             CAIRO_SET_SOURCE_RGB (cr_txt, 0);
             SmilTextBlock *b = info.first;
+            int hoff = 0;
             int voff = 0;
             while (b) {
-                const QByteArray text = b->rich_text.toUtf8 ();
-                cairo_translate (cr_txt, 0, b->rect.y() - voff);
+                cairo_translate (cr_txt, b->rect.x() - hoff, b->rect.y() - voff);
+                QTextDocument td;
+                td.setDocumentMargin (0);
+                td.setDefaultFont (b->font);
+                bool have_alpha = (s->background_color & 0xff000000) < 0xff000000;
+                QImage img (QSize (b->rect.width(), b->rect.height()), have_alpha ? QImage::Format_ARGB32 : QImage::Format_RGB32);
+                img.fill (s->background_color);
+                td.setPageSize (QSize (b->rect.width(), b->rect.height() + 10));
+                setAlignment (td, b->align);
+                td.documentLayout()->setPaintDevice (&img);
+                td.setHtml (b->rich_text);
+                QPainter painter;
+                painter.begin (&img);
+                QAbstractTextDocumentLayout::PaintContext ctx;
+                ctx.clip = QRect (QPoint (0, 0), img.size ());
+                td.documentLayout()->draw (&painter, ctx);
+                painter.end();
+
+                cairo_surface_t *src_sf = cairo_image_surface_create_for_data (
+                        img.bits (),
+                        have_alpha ? CAIRO_FORMAT_ARGB32:CAIRO_FORMAT_RGB24,
+                        img.width(), img.height(), img.bytesPerLine ());
+                cairo_pattern_t *pat = cairo_pattern_create_for_surface (src_sf);
+                cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
+                cairo_set_operator (cr_txt, CAIRO_OPERATOR_SOURCE);
+                cairo_set_source (cr_txt, pat);
+                cairo_rectangle (cr_txt, 0, 0, b->rect.width(), b->rect.height());
+                cairo_fill (cr_txt);
+                cairo_pattern_destroy (pat);
+                cairo_surface_destroy (src_sf);
+
+                hoff = b->rect.x ();
                 voff = b->rect.y ();
-                PangoLayout *layout = pango_cairo_create_layout (cr_txt);
-                pango_layout_set_width (layout, 1.0 * w * PANGO_SCALE);
-                pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
-                pango_layout_set_markup (layout, text.data (), -1);
-                pango_layout_set_font_description (layout, b->desc);
-                if (b->align > -1)
-                    pango_layout_set_alignment(layout,(PangoAlignment)b->align);
-
-                pango_cairo_show_layout (cr_txt, layout);
-
-                g_object_unref (layout);
-
                 SmilTextBlock *tmp = b;
                 b = b->next;
                 delete tmp;
@@ -1158,13 +1206,15 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
 
             // update bounds rect
             s->bounds = matrix.toUser (IRect (scr.point, ISize (w, info.voffset)));
+            txt->size = s->bounds.size;
+            txt->updateBounds (false);
 
             // update coord. for painting below
             scr = matrix.toScreen (s->bounds);
         }
     }
     IRect clip_rect = clip.intersect (scr);
-    if (!clip_rect.isEmpty ())
+    if (s->surface && !clip_rect.isEmpty ())
         paint (&txt->transition, txt->media_opacity, s, scr.point, clip_rect);
     s->dirty = false;
 }
@@ -1669,7 +1719,6 @@ void MouseVisitor::surfaceEvent (Node *node, Surface *s) {
     int rx = scr.x(), ry = scr.y(), rw = scr.width(), rh = scr.height();
     const bool inside = x > rx && x < rx+rw && y > ry && y< ry+rh;
     const bool had_mouse = s->has_mouse;
-    s->has_mouse = inside;
     if (deliverAndForward (node, s, inside, true) &&
             (inside || had_mouse) &&
             s->firstChild () && s->firstChild ()->node) {
